-- TODO

-- Why do I get two death msgs?
-- When rake didn't spawn a timer, hotman got all the way to myspellbar.  Why
--   didn't it cast?  Was it tagged as a failure?

-- Miss to Venture Co. Strip Miner comes in with name as "Venture Co", leading
--   to an incorrect bar being started.  Who generates that name?  This appears
--   to be a bug with ParserLib.  Report it.

--embedded libs
local parser = ParserLib:GetInstance("1.1")	

HotMan = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0", "AceDB-2.0", "CandyBar-2.0")
local surface = AceLibrary("Surface-1.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local paint = AceLibrary("PaintChips-2.0")

local DEBUG = nil

--------------------------------------------------------------------------------
--
-- OPTIONS
--
--------------------------------------------------------------------------------
optionsTable = {
	type = "group",
	args = {
		header = {
			type = "header",
			name = "HotMan",
			order = 1,
		},
		anchor = {
			type = "execute",
			name = "Show/Hide Anchor",
			desc = "Show/Hide the bars anchor (can also left-click a statusbar)",
			func = function() 
				if HotManFrame:IsVisible() then 
					HotManFrame:Hide()
				else 
					HotManFrame:Show()
				end 
			end,
			order = 2,
		},
		bars = {
			type = "group",
			name = "Bar",
			desc = "Statusbar options",
			order = 5,
			args = {
				scale = {
					type = "range",
					name = "Scale",
					desc = "Change statusbar scale",
					set = function(v) HotMan.db.profile.scale = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.scale end,
					min = 0.5, max = 2, step = 0.1,
					order = 1,
				},
				width = {
					type = "range",
					name = "Width",
					desc = "Change statusbar width",
					set = function(v) HotMan.db.profile.width = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.width end,
					min = 50, max = 300, step = 10,
					order = 2,
				},
				height = {
					type = "range",
					name = "Height",
					desc = "Change statusbar thickness",
					set = function(v) HotMan.db.profile.height = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.height end,
					min = 10, max = 40, step = 2,
					order = 3,
				},
				texture = {
					type = "text",
					name = "Texture",
					desc = "Statusbar textures",
					set = function(v) HotMan.db.profile.texture = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.texture end,
					validate = surface:List(),
					order = 4,
				},
				fontsize = {
					type = "range",
					name = "Font size",
					desc = "Change statusbar font size",
					set = function(v) HotMan.db.profile.fontsize = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.fontsize end,
					min = 8, max = 20, step = 0.2,
					order = 5,
				},
				fadetime = {
					type = "range",
					name = "Fade time",
					desc = "Time for a bar to fade out once the HoT/DoT has expired",
					set = function(v) HotMan.db.profile.fadetime = v HotMan:UpdateActiveBars() end,
					get = function() return HotMan.db.profile.fadetime end,
					min = 0, max = 10, step = 0.5,
					order = 6,
				},
				test = {
					type = "execute",
					name = "Test",
					desc = "Start a test bar",
					func = function() HotMan:StartBar("Testing", 30, "Interface\\Icons\\Ability_ThunderBolt", "lightgrey") end,
					order = 7,
				},
			},
		},
		incSpellNames = {
			type = "toggle",
			name = "Show spell names",
			desc = "Also show spell name in timer bar",
			get = function() return HotMan.db.profile["incSpellNames"] end,
			set = function(v) HotMan.db.profile["incSpellNames"] = v end,
			order = 8,
		},
		debug = {
			type = "toggle",
			name = "Run in debugging mode",
			desc = "Shows state information in console for debugging purposes.  It's just like spam!",
			get = function() return HotMan.db.profile.debug end,
			set = function(v) HotMan.db.profile.debug = v end,
			order = 8,
		},		
		talents = {
			type = "execute",
			name = "List talents affecting HotMan",
			desc = "List talents that affect the duration of HoTs and DoTs",
			func = function() 
				HotMan:listTalents()
			end,
			order = 9,
		},
	},
}

function HotMan:Options()
	dewdrop:Open( HotManFrame,
		'children', function() dewdrop:FeedAceOptionsTable(optionsTable) end,
		'cursorX', true, 'cursorY', true )
end

--------------------------------------------------------------------------------
--
-- Cast spells
--
--------------------------------------------------------------------------------
HotManSelectedSpell = {
--	id = nil,
	name = nil,
	rank = nil,
	target = nil,
	targetAtCast = nil,
	amount = nil,
	
	setData = function (self, name, rank, target, targetAtCast)
		if rank then rank = tonumber(rank) end

--		self.id = id;
		self.name = name;
		self.rank = rank;
		self.target = target;
		self.targetAtCast = targetAtCast;
		self.amount = 0;
	end,

	clear = function(self)
--		self.id = nil;
		self.name = nil;
		self.rank = nil;
		self.target = nil;
		self.targetAtCast = nil;
		self.amount = 0;
	end,
	
	getText = function(self)
		if hotIsTargeted(self.name) then
			t = self.target
		else
			t = self.name
		end
		
		if HotMan.db.profile["incSpellNames"] then
			t = t .. " - " .. self.name
		end
		return t
	end,
};

--------------------------------------------------------------------------------
--
-- HotMan basic management
--
--------------------------------------------------------------------------------
function HotMan:OnInitialize()
	self:RegisterDB("HotManDB")
	self:RegisterDefaults("profile", {
		texture = "BantoBarReverse",
		fontsize = 10,
		width = 200,
		height = 16,
		scale = 1,
		fadetime = 3,
		incSpellNames = nil,
		debug = false,
	} )

	self:BulkWork()
	self:RegisterChatCommand({ "/hotman", "/hm" }, optionsTable)

	self.EVENT_DELAY = 0.3
end

function HotMan:OnEnable()
	self:AddHots()

	-- chat msgs
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", HotMan.OnCombatEvent)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_SELF_BUFF", HotMan.OnCombatEvent)
 
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", HotMan.OnCombatEvent)

	-- Death
	parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_HOSTILE_DEATH", HotMan.OnDeathEvent)
	--parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", HotMan.OnDeathEvent)
	
	-- Misses
	parser:RegisterEvent("HotMan", "CHAT_MSG_COMBAT_SELF_MISSES", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF", HotMan.OnCombatEvent)
	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_SELF_DAMAGE", HotMan.OnCombatEvent)

 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_BREAK_AURA", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_SELF", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_PARTY", HotMan.OnAuraGone)
 	parser:RegisterEvent("HotMan", "CHAT_MSG_SPELL_AURA_GONE_OTHER", HotMan.OnAuraGone)
  
	-- casting
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "StartHOTFromEvent")
	--self:RegisterEvent("UNIT_SPELLCAST_START", HotMan.RecordSpellCastStart)
	self:RegisterEvent("UNIT_SPELLCAST_SENT", HotMan.RecordSpellCastSent)
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", HotMan.SpellCastFail)
	
	-- This is just dangerous -- what exactly does it do?
	--self:RegisterEvent("UNIT_SPELLCAST_FAILED", HotMan.SpellCastFail)

	--self:SecureHook("UseAction")
	--self:SecureHook("CastSpell")
	--self:SecureHook("CastSpellByName")
	--self:SecureHook("SpellTargetUnit")
	--self:SecureHook("TargetUnit", "SpellTargetUnit")
	--self:HookReport()
	
	-- Talent handling
	self:buildTalentHandlers()
	self:walkTalents()
	
	-- Handle enabling/disabling spells
	self:buildSpellEnablers(optionsTable)
	
	self.hots = {}
	self.misses = {}
	self.next_cast_is_failure = nil
end

--------------------------------------------------------------------------------
--
-- PARSING MSGS
--
--------------------------------------------------------------------------------
function HotMan:OnDeathEvent(info)
	local self = HotMan

	target = self:VictimName(info.victim)
	self:debugPrint( "Death: victim=" .. info.victim.." target="..target)
	self:ClearBars(target)
end


function HotMan:OnCombatEvent(info)
	local self = HotMan

	-- Deal with skills that are poorly formed or aren't hots
	if not info.skill then return end
	hot = getHOT(info.skill) 
	if not hot then
		--self:debugPrint( "Skill " .. boolString(info.type) .. " - " .. boolString(info.skill) .." isn't a hot")
		return
	end
	
	self:debugPrint( "Skill " .. boolString(info.type) .. " - " .. boolString(info.skill) .." from ".. boolString(info.source) .." to ".. boolString(info.victim))
	if info.type == "heal" and info.source == ParserLib_SELF then
		--self:Print( "I healed " .. info.victim .." for ".. info.amount .." from ".. info.skill .." and dotP is "..boolString(info.isDOT))
		--  if info.isDOT then
		-- This code doesn't work in all cases, needs to be disabled
		--target = self:VictimTarget(info.victim)
		--if info.victim and UnitExists(info.victim) then
		--	overheal = max(info.amount - (UnitHealthMax(target) - UnitHealth(target)), 0)
		overheal = 0
		self:UpdateHOT(info, info.amount - overheal)
		--end
		--		end
	elseif info.type == "hit" and info.isDOT and info.source == ParserLib_SELF then
		--ChatFrame1:AddMessage( "I DOTed " .. info.victim .." for ".. info.amount .." using ".. info.skill)
		self:UpdateHOT(info, info.amount)
	elseif info.type == "miss" and info.source == ParserLib_SELF then
		target = self:VictimName(info.victim)
		self:debugPrint( "Miss "..boolString(info.skill).." "..boolString(info.source).." "..boolString(info.victim).." "..boolString(target) )
		self.misses[getBarName( target, info.skill )] = true
		self:printMisses("OnCombatEvent "..target)
		--self:ClearBars( , info.skill, true )
	--elseif info.source == ParserLib_SELF then
	--elseif info.source == ParserLib_SELF or info.victim == ParserLib_SELF then
	else
		-- Deal with the situation where we need to fire off a new ability, etc
		if hot.reactive then
			self:debugPrint( "Reactive skill " .. info.type .. " - " .. boolString(info.skill) .." from ".. boolString(info.source) .." to ".. boolString(info.victim) .. " isDOT " .. boolString(info.isDOT))
			-- This thing is known to be a HOT
			barName = getBarName( self:VictimName(info.victim), info.skill )

			--for key,value in pairs(self.hots) do
			--	self:Print("HOT:",key)
			--end

			--if not self.hots[barName] then
				-- Spell isn't tracked, fire off a new bar for a reactive ability
				--self:Print( "  -> Not currently tracked, adding", badName)
				HotManSelectedSpell:setData(info.skill, 0, self:VictimName(info.victim), self:VictimName(info.victim));
				self:StartHOT(true)
			--else
				-- spell is being tracked, so update the HOT
				-- self:UpdateHOT(info, info.amount)
			--end
		end
	end
end

function HotMan:printMisses(site)
	local self = HotMan
	self:debugPrint("MISSES at "..site)
	for key,value in pairs(self.misses) do
		self:debugPrint("  * Miss:", key, value)
	end
end


function HotMan:VictimTarget( victim )
	if victim == ParserLib_SELF then 
		return "player" 
	else 
		return victim
	end
end

function HotMan:VictimName( victim )
	if victim == ParserLib_SELF then 
		return UnitName("player") 
	else 
		return victim
	end
end

function HotMan:OnAuraGone(info)
	local self = HotMan 
	self:ClearBars(self:VictimName(info.victim), info.skill)
end

function HotMan:UpdateHOT(info, amount)
	target = self:VictimName(info.victim)
	barName = getBarName( target, info.skill )
	if self.hots[barName] then
		-- spell is being tracked
		spell = self.hots[barName]
		spell.amount = spell.amount + amount
		local hot = getHOT(spell.name)
		if hot.harmful then symbol = "-" else symbol = "+" end
		text = spell:getText()
		self:SetCandyBarText(barName, text.." "..symbol..spell.amount)
--	elseif getHOT(info.skill) then
--		self.Print("UpdateHOT -- Detected Hot but it's not in the list of hots: "..barName.." target: "..boolString(info.target))
	end
end

function HotMan:ClearBars(target, skill)
	if skill then
		-- remove a single skill
		--self:Print("Removing: "..target.."  "..skill)
		barName = getBarName( target, skill )

		if self.hots[barName] then
 			self:StopCandyBar(barName)
		end
	else
		-- Removes bars corresponding to a target
		for barName,spell in pairs(self.hots) do
			if spell.target == target then
				self:StopCandyBar(barName)
			end
		end
	end
end

--------------------------------------------------------------------------------
--
-- SETUP FUNCTIONS
--
--------------------------------------------------------------------------------

function HotMan:BulkWork()
	self.textures = { }
	for name, texture in pairs(surface:Iterate()) do
		self.textures[name] = texture
	end

	self.activebars = { }

	paint:RegisterColor("grey", 0.3, 0.3, 0.3)
	paint:RegisterColor("lightgrey", 0.6, 0.6, 0.6)
	
	self:CreateBackbone()
end

--------------------------------------------------------------------------------
--
-- ADDING TIMER BARS
--
--------------------------------------------------------------------------------

function getBarName(target, name)
	if hotIsTargeted(name) then
		return target.." -- "..name
	else
		return name
	end
end

-- unit aura change
--function HotMan:StartHOTFromCast()
--	if HotManHOTs[HotManSelectedSpell.name] then
--		local hot = getHOT(HotManSelectedSpell.name)
--		if hot.targeted then
--			-- untargeted spells are handled as chat events
--			self:StartHOT()
--		end
--	end
--end


function HotMan:StartHOTFromEvent(targetArg, spellArg, rankArg, immediate)
	self:debugPrint("Spell cast event "..boolString(targetArg).." "..boolString(HotManSelectedSpell.name).." "..boolString(HotManSelectedSpell.rank))
	hot = getHOT(HotManSelectedSpell.name)
	if hot and not hot.reactive then
		self:StartHOT(false)
	end
end

function HotMan:StartHOT(immediate)
	self:debugPrint("StartHOT Called "..HotManSelectedSpell.name.." "..HotManSelectedSpell.rank)
	hot = getHOT(HotManSelectedSpell.name)
	if hot then
		if self.db.profile[hot.name] then
			if hot.harmful then
				--self:Print("StartHOT: harmful "..hot.name)
				color = "red" 
			else 
				--self:Print("StartHOT: helpful")
				color = "green" 
			end
			
			self:debugPrint("StartHOT: "..HotManSelectedSpell.name.."  rank "..HotManSelectedSpell.rank.."  target  "..HotManSelectedSpell.target)

			if HotManSelectedSpell.target == "targeting" then
				HotManSelectedSpell.target = HotMan:DetermineTarget(true)
			end
	
			if not HotManSelectedSpell.target and hotIsTargeted(HotManSelectedSpell.name) then
				-- Handle the situation where the spell really is untargeted
				-- i.e., vanish, stealth
				self:debugPrint("  -> not trigging, spell has no target")
				return
			end
	
			if not hotIsTargeted(HotManSelectedSpell.name) then
				HotManSelectedSpell.target = UnitName("player")
			end
	
			barName = getBarName(HotManSelectedSpell.target, HotManSelectedSpell.name)
	
			-- start bar and remember info about the spell
			self.hots[barName] = tablecopy(HotManSelectedSpell)
			
			duration = hot:getDuration(HotManSelectedSpell.rank)
			if immediate then
				self:debugPrint("  -> Starting bar immediately")
				self:StartBar(barName, duration, hot.texture, color, nil, nil, HotManSelectedSpell:getText() )
			else
				self:debugPrint("  -> Spawning MyStartBar")
				self:ScheduleEvent( MyStartBar, self.EVENT_DELAY, barName, duration, hot.texture, color, HotManSelectedSpell:getText() )
			end
			HotManSelectedSpell:clear()
		end
	end
end

function MyStartBar( barName, duration, texture, color, text )
	local self = HotMan

	--self:Print("MyStartBar misses", barName)
	self:printMisses("MyStartBar "..barName)
	if self.misses[barName] then
		self:debugPrint("  "..barName.." MISSED PREVIOUSLY")
	elseif self.next_cast_is_failure then
		self:debugPrint("  "..barName.." LAST CAST WAS FAILURE") 
	else
		self:debugPrint("  *** Starting bar "..boolString(barName)) 
		self:StartBar(barName, duration - self.EVENT_DELAY, texture, color, nil, nil, text )
	end
	self.misses = {}
	self.next_cast_is_failure = nil
end

function HotMan:DetermineTarget(ForceResolve)
	-- determine the target
	id = "BUG"
	if SpellIsTargeting() and not ForceResolve then
		--self:Print("SpellIsTargeting: "..boolString(SpellCanTargetUnit("target")))
		return "targeting"
	elseif IsActionSelfCastKeyDown() then
		id = "player"
	elseif UnitName("target") and (not ForceResolve or UnitName("target") ~= HotManSelectedSpell.targetAtCast) then
		id = "target"
	elseif UnitName("mouseover") then
		id = "mouseover"
	elseif ForceResolve then
		-- we assume the spell is going to the player, as mouseover is false 
		-- when selecting the player
		id = "player"
	else
		-- Special case where the spell is untargeted -- think Stealth for rogues
		return nil
	end
	target = UnitName(id)
	--self:Print("  Target -> "..target.." Targeting? -> "..boolString(SpellIsTargeting()).."  ForceResolve -> "..boolString(ForceResolve).."  CanTarget? -> "..boolString(SpellCanTargetUnit(id)))
	--self:Print("  Target -> "..UnitName("mouseover"))
	return target
end

--------------------------------------------------------------------------------
--
-- GUI BAR FUNCTIONS
--
--------------------------------------------------------------------------------

-- creates the statusbar anchor and registers candybar group
function HotMan:CreateBackbone()
	self.mainframe = CreateFrame("Button", "HotManFrame", UIParent)
	self.mainframe:SetHeight(16)
	self.mainframe:SetWidth(200)
	self.mainframe:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	                             tile = true, tileSize = 16,
	                             insets = { left = 0, right = 0, top = 0, bottom = 0 }
	                          })
	self.mainframe:SetBackdropColor(0.3, 0.3, 0.3, 0.5)
	self.mainframe:SetMovable(true)
	self.mainframe:EnableMouse(true)
	self.mainframe:RegisterForDrag("LeftButton")
	self.mainframe:RegisterForClicks("RightButtonUp")
	self.mainframe:SetScript("OnDragStart", function() this:StartMoving() end)
	self.mainframe:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing()
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			self.db.profile.x = floor(d + 0.5)
			self.db.profile.y = floor(e + 0.5)
		end
	end)
	self.mainframe:SetScript("OnClick", function()
		if arg1 == "RightButton" then
			HotMan:Options()
		end
	end)
	
	self.mainframe:Hide()
	
	self.mainframe.text = self.mainframe:CreateFontString("HotManFrameText", "OVERLAY")
	self.mainframe.text:SetFont(GameFontHighlightSmall:GetFont())
	self.mainframe.text:SetText("HotMan")
	self.mainframe.text:ClearAllPoints()
	self.mainframe.text:SetAllPoints(self.mainframe)
	
	if not self.db.profile.x then
		self.mainframe:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
	else
		self.mainframe:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	
	self:RegisterCandyBarGroup("HotManBottom")
	self:SetCandyBarGroupPoint("HotManBottom", "TOP", self.mainframe, "TOP", 0, -15)
	
	self:RegisterCandyBarGroup("HotManTop")
	self:SetCandyBarGroupPoint("HotManTop", "BOTTOM", self.mainframe, "BOTTOM", 0, 15)
	self:SetCandyBarGroupGrowth("HotManTop", true)
end

-- begin a statusbar timer
-- - name: id of bar and text of bar
-- - duration: full duration of bar
-- - icon: icon location
-- - color: paintchips color
-- - nonactive: set true if you want to continue this bar timer after zoning
-- - separate: set true to anchor to the top instead of bottom
-- - specialText: bar text if different from name
function HotMan:StartBar(name, duration, icon, color, nonactive, separate, specialText)
	--self:Print("Starting bar "..name)

	local self = HotMan
	if not specialText then
		self:RegisterCandyBar(name, duration, name, icon, color)
	else
		self:RegisterCandyBar(name, duration, specialText, icon, color)
	end
	self:SetCandyBarTexture(name, self.textures[self.db.profile.texture] or self.textures.default)
	self:SetCandyBarWidth(name, self.db.profile.width)
	self:SetCandyBarHeight(name, self.db.profile.height)
	self:SetCandyBarFontSize(name, self.db.profile.fontsize)
	self:SetCandyBarScale(name, self.db.profile.scale)
	self:SetCandyBarFade(name, self.db.profile.fadetime)
	self:SetCandyBarBackgroundColor(name, "Grey", 0.6)
	if separate then
		self:RegisterCandyBarWithGroup(name, "HotManTop")
	else
		self:RegisterCandyBarWithGroup(name, "HotManBottom")
	end
	self:SetCandyBarOnClick(name, function(name, button) self:OnClick(name, button) end)
	self:SetCandyBarCompletion(name, function(name, button) self:OnCompletion(name, button) end, name)

	self:StartCandyBar(name, true)
	if not nonactive then
		tinsert(self.activebars, name)
	end
end

-- updates active bars with new textures if needed
function HotMan:UpdateActiveBars()
	for i=1,getn(self.activebars),1 do
		local value = self.activebars[i]
		self:SetCandyBarTexture(value, self.textures[self.db.profile.texture] or self.textures.default)
		self:SetCandyBarWidth(value, self.db.profile.width)
		self:SetCandyBarHeight(value, self.db.profile.height)
		self:SetCandyBarFontSize(value, self.db.profile.fontsize)
		self:SetCandyBarScale(value, self.db.profile.scale)
		self:UpdateCandyBarGroup("HotManBottom")
		self:UpdateCandyBarGroup("HotManTop")
	end
end

--------------------------------------------------------------------------------
--
-- BAR EVENT HANDLING
--
--------------------------------------------------------------------------------

-- gives some user friendliness to the Candy Bars
-- - left-click to toggle anchor
-- - right-click to bring up config menu
function HotMan:OnClick(name, button)
	if button == "LeftButton" then
		if HotManFrame:IsVisible() then
			HotManFrame:Hide()
		else
			HotManFrame:Show()
		end
	elseif button == "RightButton" then
		--if IsShiftKeyDown() then
		--	self:StopBar(name)
		--else
		HotMan:Options()
		--end
	end
end

function HotMan:FormatChatTime(secs)
	local math_mod = math.fmod or math.mod
	return string.format("%d:%02d", secs / 60, math_mod(secs, 60))
end

function HotMan:OnCompletion( name )
	spell = self:RemoveSpell(name)
	--self:Print("Bar completed "..name.." -- final amount "..spell.amount)
end

function HotMan:RemoveSpell( name )
	spell = self.hots[name]
	self.hots[name] = nil
	return spell
end


--------------------------------------------------------------------------------
--
-- EVENT HANDLING
--
--------------------------------------------------------------------------------
function HotMan:SpellCastFail()
	local self = HotMan
	--self:Print("SpellCastFail")
	self.next_cast_is_failure = true
end

function HotMan:RecordSpellCastStart(name, duration)
	--self:Print("RecordSpellCastStart "..name.." "..boolString(duration))
end

function HotMan:RecordSpellCastSent(name, rank, target)
	--HotMan:Print("RecordSpellCastSent "..boolString(name).." "..boolString(rank).." "..boolString(target))

	if rank == "" then
		rankN = 0
	else
		s,e,rankN = rank:find("Rank (%d+)")
	end

	--HotMan:Print("  -> PARSED "..boolString(name).." "..boolString(rankN))
	if name and rank then
		HotMan:RecordCast(name, rankN, target)
	end
end

function HotMan:RecordCast(spellName, spellRank, target)
	-- We've just finished casting, so reset the failure state
	self.next_cast_is_failure = nil

	--self:Print("RecordCast",spellName,spellRank,target)
	--HotManSelectedSpell:setData(spellName, spellRank, self:DetermineTarget(nil), UnitName("target"));
	HotManSelectedSpell:setData(spellName, spellRank, target, target);
	
	if HotManHOTs[HotManSelectedSpell.name] then
		local hot = getHOT(HotManSelectedSpell.name)
		if hot.execute then
			hot.execute(hot, HotManSelectedSpell)
		end
	end
end

--------------------------------------------------------------------------------
--
-- UTILITIES
--
--------------------------------------------------------------------------------
function tablecopy( table )
	t = {}
	for k, v in pairs(table) do t[k] = v end
	return t
end

function boolString( bool )
	if bool then return bool else return "false" end
end

-- print chat msg
function HotMan:debugPrint(msg)
	if self.db.profile.debug then
		self:Print(msg)
	end
end
