--------------------------------------------------------------------------------
--
-- HOTS/DOTS
--
--------------------------------------------------------------------------------
HotManHOT = {
	name = nil,
	rank = nil,
	duration = nil,
	harmful = nil,
	targeted = nil,
	texture = nil,
}

function HotManHOT:new(o)
	o = o or {}   -- create object if user does not provide one
	setmetatable(o, self)
	self.__index = self
	return o
end

function HotManHOT:getDuration(rank)
	if not rank or not self.ranks then
		return hot.duration
	else
		--HotMan:Print("RANKS ARE: ")
		--for key,value in pairs(self.ranks) do
		--	HotMan:Print(key,type(key),value)
		--end			
		--HotMan:Print("Incoming rank is",rank,type(rank),self.ranks[rank])
		return hot.duration + self.ranks[rank]
	end
end

HotManHOTs = {}

function newHOT( name, duration, harmful, targeted, extraKeys )
	hot = HotManHOT:new({name=name, duration=duration, harmful=harmful, targeted=targeted})
	
	if extraKeys then 
		for key,value in pairs(extraKeys) do
			hot[key] = value
		end
	end
	
	if not hot.texture then
		hot.texture = findSpellTexture( hot.iconSpell or name )
		HotMan:Print("Spell "..name.." has texture "..boolString(hot.texture))
	end

	HotManHOTs[name] = hot
end

function findSpellTexture( name )
	local i = 1
	while true do
	   local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
	   if not spellName then
		  do break end
	   end

		if name == spellName then	   
	   		--HotMan:Print( "SPELL MAP: " .. name .. '->' .. i )
			return GetSpellTexture( i, BOOKTYPE_SPELL )
	   	end
	   i = i + 1
	end
	
	for i = 1, GetNumTalentTabs() do
		for j = 1, GetNumTalents(i) do
			if GetTalentInfo(i, j) == name then 
				local _, tx = GetTalentInfo(i, j)
				return tx
			end
		end
	end
end

function hotIsTargeted( name )
	local hot = getHOT( name )
	if hot then
		return hot.targeted
	else
		return nil
	end
end

function getHOT( name )
	--HotMan:Print("Getting hot: "..name.."  "..rank)
	if not HotManHOTs[name] then
		return nil
	else
		return HotManHOTs[name]
	end
end


function HotMan:buildSpellEnablers(optionsTable)
	self:Print("HotMan:buildSpellEnablers", self, optionsTable)
	
	optionsTable.args.spells = {
		type = "group",
		name = "Spells",
		desc = "Enable/Disable display of individual spells",
		args = {}
	}

	for name,hot in pairs(HotManHOTs) do
		if hot.texture then
			if self.db.profile[name] == nil then
				self.db.profile[name] = true
			end
			
			option = {
				type = "toggle",
				name = name,
				desc = name,
				get = function() return self.db.profile[name] end,
				set = function(v) self.db.profile[name] = v end,
			}
			optionsTable.args.spells.args[name] = option
		end
	end
end

function HotMan:AddHots()
	newHOT( "Renew", 15, nil, true )
	newHOT( "Shadow Word: Pain", 18, true, true )
	newHOT( "Fade", 10, nil, nil )
	newHOT( "Power Word: Shield", 15, nil, true )
	newHOT( "Weakened Soul", 30, nil, true, {reactive=true, texture="Interface\\Icons\\Spell_Holy_AshesToAshes"} )
	newHOT( "Starshards", 6, true, true )
	newHOT( "Symbol of Hope", 15, nil, true )
	newHOT( "Psychic Scream", 8, true, nil )
	newHOT( "Devouring Plague", 24, true, true )
	newHOT( "Elune's Grace", 15, nil, nil )
	newHOT( "Feedback", 15, true, nil )
	newHOT( "Holy Fire", 10, true, true )
	newHOT( "Mind Soothe", 15, true, true )
	newHOT( "Shackle Undead", 30, true, true )
	newHOT( "Abolish Disease", 20, nil, true )
	newHOT( "Levitate", 120, nil, nil )
	newHOT( "Vampiric Touch", 15, true, true )
	newHOT( "Shadowfiend", 15, true, nil )
	newHOT( "Mind Control", 60, true, true )
	newHOT( "Power Infusion", 15, nil, true )
	newHOT( "Silence", 5, true, true )
	newHOT( "Vampiric Embrace", 60, true, true )
	newHOT( "Pain Suppression", 8, nil, nil )
	newHOT( "Blackout", 3, true, true )
	newHOT( "Blessed Recovery", 6, nil, nil )
	newHOT( "Shadow Vulnerability", 15, true, true )
	newHOT( "Blessed Resilience", 6, nil, nil )
	newHOT( "Focused Casting", 6, nil, nil )
	newHOT( "Inspiration", 15, nil, true, {reactive=true, texture="Interface\\Icons\\INV_Shield_06"} )
	newHOT( "Surge of Light", 10, nil, nil, {reactive=true} )
	newHOT( "Spirit Tap", 15, nil, nil, {reactive=true} )
	newHOT( "Shadow Weaving", 15, nil, nil, {reactive=true} )

	-- Prayer of Mending ???	

	-- Druid spells [there has to be a better way to list these!]
	newHOT( "Moonfire", 12, true, true )
	newHOT( "Rejuvenation", 12, nil, true )
	newHOT( "Entangling Roots", 0, true, true, {ranks={12,15,18,21,24,27,27}})
	newHOT( "Regrowth", 21, nil, true )
	newHOT( "Innervate", 20, nil, true )
	newHOT( "Faerie Fire", 40, true, true )
	newHOT( "Faerie Fire (Feral)", 40, true, true )
	newHOT( "Hibernate", 0, true, true, {ranks={20,30,40}} )
	newHOT( "Tiger's Fury", 6, nil, nil )
	newHOT( "Soothe Animal", 15, true, true )
	newHOT( "Abolish Poison", 8, nil, true )
	newHOT( "Insect Swarm", 12, true, true )
	newHOT( "Bash", 0, true, true, {ranks={2,3,4}} )	-- Bear
	newHOT( "Rip", 12, true, true )	-- kitty
	newHOT( "Rake", 9, true, true )  -- kitty
	newHOT( "Pounce", 3, true, true )  -- kitty
	newHOT( "Mangle (Cat)", 12, true, true )  -- kitty
	newHOT( "Mangle (Bear)", 12, true, true )  -- bear
	newHOT( "Demoralizing Roar", 30, true, nil )  -- bear
	newHOT( "Enrage", 10, nil, nil )  -- bear
	newHOT( "Challenging Roar", 6, true, nil )  -- bear
	newHOT( "Frenzied Regeneration", 10, nil, nil )  -- bear
	newHOT( "Barkskin", 15, nil, nil )  -- Tree?
	newHOT( "Lifebloom", 7, nil, true )  -- Tree
	newHOT( "Lacerate", 15, true, true )  -- Tree?
	newHOT( "Feral Charge", 4, true, true )  -- Tree?
	newHOT( "Clearcasting", 15, nil, nil, {reactive=true, texture="Interface\\Icons\\Spell_Shadow_ManaBurn"} )
	newHOT( "Pounce Bleed", 18, true, true, {reactive=true, iconSpell="Pounce"} )
	newHOT( "Starfire Stun", 3, true, true, {reactive=true, iconSpell="Starfire"} )
	newHOT( "Force of Nature", 30, true, nil )
	newHOT( "Maim", 0, true, true, {execute=DetermineMaimDuration})
end

function DetermineMaimDuration( hot, spell )
	hot.duration = GetComboPoints() + 1
	--HotMan:Print("DetermineMaimDuration: "..hot.duration)
end